/*********************************************
* Author: Eric Z. Ayers
*
* This program is to test the leds connected to a ULN2003A driver
* for the Faux candles project.
*
* High signal enables the LED, Low signal turns the LED off.
*
* PB6 - LED0
* PB5 - LED1
* PB4 - LED2
* PB3 - LED3
* PA4 - LED4
* PA5 - LED5
* PA6 - LED6
*
* Debug LED
* PA7 - DEBUG_LED  (with a 220uf cap and a 1K resistor)
*
* Chip type           : AT TINY 26
* Clock frequency     : 1.000000 MHz
*
*********************************************/

/*************************************************************************
 *			Macros
 *************************************************************************/


/* Set CPU frequency for libc routines and macros */
#define F_CPU 1000000UL  // 1 MHz
//#define F_CPU 14.7456E6
//#define F_CPU 4000000UL  // 4 MHz

#define LED0_PIN PB3
#define LED1_PIN PB4
#define LED2_PIN PB5
#define LED3_PIN PB6
#define LED4_PIN PA4
#define LED5_PIN PA5
#define LED6_PIN PA6

/*************************************************************************
 *			Include Files
 *************************************************************************/

#include <io.h>
#include <avr/delay.h>

/*************************************************************************
 *			Static Data
 *************************************************************************/


/*************************************************************************
 *			Function Prototypes
 *************************************************************************/
static void delay_ms (uint16_t ms);
static void init_leds ( void );
static void set_all_leds (uint8_t value);
static void set_led (int8_t led_number, uint8_t value);

/*************************************************************************
 *			Functions
 *************************************************************************/

/* Delay using the avr libc delay functions */
static void delay_ms (uint16_t ms)
{
	for (;ms>0;ms--)
		_delay_ms(1);
}

/*
  Set the LED pins as outputs for their respective ports.
  Turn off all LEDs.
 */
static void init_leds ( void )
{
	/* Set the pins for LEDs to be outputs */
	DDRB |= _BV(LED0_PIN) | _BV(LED1_PIN) | _BV(LED2_PIN) | _BV(LED3_PIN);
	DDRA |= _BV(LED4_PIN) | _BV(LED5_PIN) | _BV(LED6_PIN);
	
	/* Set the output values for the pins to low */
	set_all_leds (0);
}

/*
  Set all LEDS to the same value
   value:    > 0  set all LEDS on,  0 set all LEDS off
*/
static void set_all_leds (uint8_t value)
{
	if (value > 0)
	{
		PORTB |= _BV(LED0_PIN) | _BV(LED1_PIN) | _BV(LED2_PIN) |
			_BV(LED3_PIN);
		PORTA |= _BV(LED4_PIN) | _BV(LED5_PIN) | _BV(LED6_PIN);
	}
	else
	{
		PORTB &= ~(_BV(LED0_PIN) | _BV(LED1_PIN) | _BV(LED2_PIN)
			   | _BV(LED3_PIN));
		PORTA &= ~(_BV(LED4_PIN) | _BV(LED5_PIN) | _BV(LED6_PIN));
	}
}

/*
  Turn a single LED on or off
  
  led_number:  pass a number 0 - 6 to set the specified LED
  value:       > 0 Turn LED on, 0 turn LED off
*/
static void set_led (int8_t led_number, uint8_t value)
{
#define SET_LED(port, pin) 		\
	if (value > 0) {	        \
		(port) |= _BV((pin));   \
	} else { 			\
		(port) &= ~(_BV((pin)));\
	}
	
	switch (led_number)
	{
	case 0:
		SET_LED(PORTB, LED0_PIN);
		break;
	case 1:
		SET_LED(PORTB, LED1_PIN);
		break;		
	case 2:
		SET_LED(PORTB, LED2_PIN);
		break;		
	case 3:
		SET_LED(PORTB, LED3_PIN);
		break;		
	case 4:
		SET_LED(PORTA, LED4_PIN);
		break;		
	case 5:
		SET_LED(PORTA, LED5_PIN);
		break;
	case 6:
		SET_LED(PORTA, LED6_PIN);		
		break;		
       
	}
	
#undef SET_LED
}


/* main loop of the program */
int main(void)
{
	/* INITIALIZE */
	init_leds();

	/* enable PA7 as output for DEBUG LED*/
	DDRA |= _BV(PA7);
	
	
	/* BLINK, BLINK ... */
	
	while (1)
	{
		int8_t i;
		
		delay_ms(2000);

		
		/* Blink all the leds */
		for (i=0;i<10;i++)
		{
			set_all_leds (1);
			delay_ms(150);
			set_all_leds (0);
			delay_ms(350);	
		}
		

		/* Cycle throught the leds */
		for (i=0;i<7;i++)
		{
			set_led(i, 1);
			delay_ms(500);
			set_led(i, 0);
		}

		/* with all off, turn on in series */
		for (i=0;i<7;i++)
		{
			set_led(i, 1);
			delay_ms(500);
		}
		
		/* Leave all on and turn off in series */
		for (i=0;i<7;i++)
		{
			set_led(i, 0);
			delay_ms(500);
		}		

	}
}

