/*********************************************
* Author: Eric Z. Ayers
*
* fauxcandles - Controller for halloween display 2005
*
* See http://www.parallexefx.com/  APPNOTE 00001 Faux Candles
*
* This program controls:
*   6 LED candles as described in the ParallexEFX app note mentioned above.
*   An IR photodetector on one pin to detect the presence of an
*     unsuspecting trick-or-treater breaking a flashlight beam.
*   A relay attached to one output pin to control a UV light
*   A remote control unit attached to another output pin.
*
* At startup, the program goes to the FLICKER state.
*
* During the FLICKER state, the 6 candles flicker, the relay is open,
* and the remote control output pin is low. The next state is TRIGGERED.
*
* In the TRIGGERED state, the candles start a different sequence.
* Instead of flickering they all fade, and then  glow on and off all
* together a few times to create the expectation that something
* is about to happen.  This sequence lasts a few seconds.  Next state
* is SPOOK.
*
* In the SPOOK state, the relay closes to turn on the UV light.
* The remote control pin goes high to signal the "spook on a rope".
* The candles flash in sequence to create a feeling of chaos.
* This sequence lasts for 20 seconds, the time it takes for the spook
* to do its stuff.  Next state is RECOVER.
*
* In the RECOVER state, the candles go back to flickering, but the
* beam detector is not sampled. This is to allow the trick-or-treater
* to get their candy in relative peace.  The next state is FLICKER
*
* 6 leds are connected to a ULN2003A driver
* for the Faux candles project.  Each LED is wired with a current limiting
* resistor and a 470uf capacitor.
*
* The relay is connected to the other ULN2003A driver output.
*
* High signal enables the LED, Low signal turns the LED off.
*
* PB3 - LED0
* PB4 - LED1
* PB5 - LED2
* PB6 - LED3
* PA4 - LED4
* PA5 - LED5
* 
*
* Relay coil - To turn black light on and off
* PA6  High == Relay Open (normal position)   Low == Relay Closed  
*
* Flashlight beam to IR detector diode
* PA0 - Read with ADC
*
* Debug LED
* PA7 - DEBUG_LED  (with a 220uf cap and a 1K resistor)
*
* Chip type           : AT TINY 26
* Clock frequency     : 8.000000 MHz
*
*********************************************/

/*************************************************************************
 *			Macros
 *************************************************************************/


/* Set CPU frequency for libc routines and macros */
//#define F_CPU 1000000UL  // 1 MHz
//#define F_CPU 14.7456E6
//#define F_CPU 4000000UL  // 4 MHz
//#define F_CPU 8000000UL  // 8 MHz
#define F_CPU 8.0E6  // 8MHz

#define NUM_LEDS	6
#define LED0_PIN 	PB3
#define LED1_PIN 	PB4
#define LED2_PIN 	PB5
#define LED3_PIN 	PB6
#define LED4_PIN 	PA4
#define LED5_PIN 	PA5


/* These parameters control the flickering behavior */
#define DELAY_MS_RANGE 70   // Range of values created
#define MIN_LED_ON       1    // Minimum to leave the LED on
#define MIN_LED_OFF   100  // Minimum time to leave LED off

/* This macro determines how long to wait before starting the
   flicker sequence.
 */
#define FLICKER_STATE_INIT_DELAY_SECS  2

/* This macro determines how long we stay in the 'spook' state */
#define SPOOK_STATE_SECS 20

/* This macro determines how long we stay in the 'recover' state */
#define RECOVER_STATE_SECS 60

/* Macros to control the output devices */
#define DEBUG_LED_ON	PORTA |= _BV(PA7);
#define DEBUG_LED_OFF	PORTA &= ~(_BV(PA7));
#define RELAY_CLOSED 	PORTA &= ~(_BV(PA6))
#define RELAY_OPEN      PORTA |= _BV(PA6)
#define REMOTE_ON       PORTA |= _BV(PA2)
#define REMOTE_OFF      PORTA &= ~(_BV(PA2))

// Any reading below this from the ADC,  interpret as someone's nearby
#define BEAM_TRIP_POINT  180 /* .5 volts */
#define MIN_BEAM_TRIPS   50  /* you must trip the beam for 50 ms */

/*************************************************************************
 *			Include Files
 *************************************************************************/

#include <io.h>
#include <stdlib.h>
#include <avr/delay.h>
#include <avr/signal.h>
#include <avr/interrupt.h>

/*************************************************************************
 *			Static Data
 *************************************************************************/
static	int16_t	delay_led_secs[NUM_LEDS];  /* Array containing the # of secs
					      to delay.
					      If # secs > 0, led  is on.  
					      If # secs < 0, led  is off.
					   */
static int num_consecutive_trips;



/*************************************************************************
 *			Function Prototypes
 *************************************************************************/

static void delay_ms (uint16_t ms);
static void delay_us (int32_t us);
static void init_adc (void);
static void init_leds (void);
static void set_all_leds (uint8_t value);
static void set_led (int8_t led_number, uint8_t value);
static uint16_t do_adc (int8_t pin_number);
static int8_t check_beam_sensor (void);


	
/*************************************************************************
 *			Functions
 *************************************************************************/

/*
  delay_ms
  --------
  Delay up to 2^16 milliseconds using the avr libc delay functions
*/
static void delay_ms (uint16_t ms)
{
	for (;ms>0;ms--)
		_delay_ms(1);
}


/* init_adc
   --------
 */
static void init_adc (void )
{
	/* Turn on the ADC */
	ADCSR |= _BV(ADEN);

	/* Set the ADC prescaler /32 */
	ADCSR = 0x85;
}

/*
  do_adc
  ------
  Peform a single adc conversion, flip the adc off before exiting.
  The ADC is 10 bit, so we will get a value between 0 and 1023

  pin_number: ADC pin to read.  Pass a value between 0 & 7
 */
static uint16_t do_adc (int8_t pin_number )
{
	
	/* Configure to A/D Converter for ADC0 input, AVCC 
	   right adjust.
	    SEL MUX
	   0b00 00 <xxx> 
	   <xxx> valid pin numbers are 0-7 
	*/
	DDRA &=~(_BV(pin_number));
	ADMUX = (pin_number);

	/* Start the conversion */
	ADCSR |= _BV(ADSC);
	
	/* Wait for the conversion to finish */
	while (!(ADCSR & _BV(ADSC)))
		; /* do nothing */

	/* Our result is waiting for us in ADCL and ADCH */
	return (ADCL + (((uint16_t)ADCH) <<8));

}


/*
  init_leds
  ---------
  Set the LED pins as outputs for their respective ports.
  Turn off all LEDs.
 */
static void init_leds ( void )
{
	/* Set the pins for LEDs to be outputs */
	DDRB |= _BV(LED0_PIN) | _BV(LED1_PIN) | _BV(LED2_PIN) | _BV(LED3_PIN);
	DDRA |= _BV(LED4_PIN) | _BV(LED5_PIN);
	
	/* Set the output values for the pins to low */
	set_all_leds (0);

}

/*
  set_all_leds
  ------------
  Set all LEDS to the same value
  
  value:    > 0  set all LEDS on,  0 set all LEDS off
  
*/
static void set_all_leds (uint8_t value)
{
	if (value > 0)
	{
		PORTB |= _BV(LED0_PIN) | _BV(LED1_PIN) | _BV(LED2_PIN) |
			_BV(LED3_PIN);
		PORTA |= _BV(LED4_PIN) | _BV(LED5_PIN);
	}
	else
	{
		PORTB &= ~(_BV(LED0_PIN) | _BV(LED1_PIN) | _BV(LED2_PIN)
			   | _BV(LED3_PIN));
		PORTA &= ~(_BV(LED4_PIN) | _BV(LED5_PIN));
	}
}

/*
  set_led
  -------
  Turn a single LED on or off
  
  led_number:  pass a number 0 - 6 to set the specified LED
  value:       > 0 Turn LED on, 0 turn LED off
*/
static void set_led
(
	int8_t led_number,
	uint8_t value
)
{
#define SET_LED(portnum, pin) 		   \
	if (value > 0) {	           \
		(portnum) |= _BV((pin));   \
	} else { 			   \
		(portnum) &= ~(_BV((pin)));\
	}
	
	switch (led_number)
	{
	case 0:
		SET_LED(PORTB, LED0_PIN);
		break;
	case 1:
		SET_LED(PORTB, LED1_PIN);
		break;		
	case 2:
		SET_LED(PORTB, LED2_PIN);
		break;		
	case 3:
		SET_LED(PORTB, LED3_PIN);
		break;		
	case 4:
		SET_LED(PORTA, LED4_PIN);
		break;		
	case 5:
		SET_LED(PORTA, LED5_PIN);
		break;
	}
	
#undef SET_LED
} /* end set_led () */

/*
   check_beam_sensor
   -----------------

   The sensor samples the output of the sensor with the ADC which
   returns a value between 0 an 1024.  0 == 0V, 1024 == 5V.
   

   returns: 1 if the proximity sensor detects something 
            0 if not.
   
 */
static int8_t check_beam_sensor ( void )
{
	static int16_t
		num_beam_trips = 0;
/* ------------------------------------------------------------------------- */
	
	int16_t adc_result = do_adc(PA0);

	/* Debounce the input - make sure we don't trigger on some
	   kind of noise.
	*/
	if (adc_result < BEAM_TRIP_POINT)
	{
		num_beam_trips++;
	}
	else
	{
		num_beam_trips = 0;
	}

	if (num_beam_trips >= MIN_BEAM_TRIPS)
	{
		/* reset for next time */
		num_beam_trips = 0;
		return 1;
	}
	
	return 0;
	
} /* end check_beam_sensor */


/* flicker_leds
   -----------
   Simulate candle flicker on LED0
   LED0 is connected with a 470UF capacitor
   before the current limiting resistor.
*/
void flicker_leds (void)
{
	uint16_t count;
/* ------------------------------------------------------------------------ */
	
	/* Loop through each LED */
	for (count=0;count<NUM_LEDS;count++)
	{
		if (delay_led_secs[count] > 0)
		{
			/* The LED is currently on.  */
			set_led(count, 1); 
			delay_led_secs[count]--;
			
			if (delay_led_secs[count]==0)
			{
				/* Set the delay for the led to be
				   off
				*/
				delay_led_secs[count]=
					-(((random() % DELAY_MS_RANGE) * 2) + MIN_LED_OFF);
				/* Turn off led */
				set_led(count, 0);
				
				/* go to the top of the loop */
				continue;
			}
		}
		
		
		else if (delay_led_secs[count] < 0)
		{
			/* the LED is currently off */
			set_led(count, 0); 
			delay_led_secs[count]++;
			
			if (delay_led_secs[count] == 0)
			{
				/* Generate a random value
				   between MIN and (MAX_DELAY +MIN)
				*/
				delay_led_secs[count]=
					(random() % DELAY_MS_RANGE)
					+MIN_LED_ON;
				
				/* Turn on led */
				set_led(count, 1); // turn on led
				
				/* go to the top of the loop
				*/
				continue;
				
			}
		}
		
		
	} /* end for each LED */
	
} /* end flicker_leds() */


/*
  flicker_state
  -------------
  This makes the candles flicker and waits for someone to trip the
  proximity sensor.

  DEBUG LED is OFF.
  
 */
static void flicker_state (void )
{
	int16_t count;
	
	RELAY_OPEN;
	REMOTE_OFF;
	DEBUG_LED_OFF;


	/* Init the LED delay array to -1 to turn them on after the first loop.
	 */
	for (count=0;count<NUM_LEDS;count++)
	{
		delay_led_secs[count]=-1;
	}
	
	
	/* let the timer get a few samples. Delay for 4 seconds */
	count = 250 * FLICKER_STATE_INIT_DELAY_SECS;
	while (count-- > 0)
	{
		/* Change the flashing of the LEDs  */
		flicker_leds();
		DEBUG_LED_ON;
		delay_ms(1);
		DEBUG_LED_OFF;
		flicker_leds();
		delay_ms(1);
		flicker_leds();
		delay_ms(1);
		flicker_leds();
		delay_ms(1);
		flicker_leds();
		delay_ms(1);		
	}
	
	delay_ms(1000);
	
	while (1)
	{

		if (check_beam_sensor())
		{
			/* There must be something out there.
			   Start the spooky sequence.
			*/
			break;
			set_all_leds(0);
			DEBUG_LED_ON;
			delay_ms(2000);
			DEBUG_LED_OFF;
		}
		
		/* Change the flashing of the LEDs  */
		flicker_leds();
		
		delay_ms(1);
	}

	
} /* end flicker_state () */


/*
  triggered_state
  ---------------
  In this state, close the relay to turn on the UV light and
  send a signal to the remote control to start the spook on a rope.
 */
static void triggered_state (void )
{
	int8_t count;
	
	DEBUG_LED_ON;
	/* This state trips off the spook on a rope and
	   illuminates it.
	*/
	RELAY_CLOSED;
	
	/* Fire the remote control output to start the
	   spook on a rope
	*/
	REMOTE_ON;
	
	/* Run the remote for 1 second.  This is way
	   overkill. It is supposed to allow 9600bps
	   communication.
	   
	   Forget about the LEDs, they'll fade a little
	   bit in tandem and that will be spooky enough.
	*/
	/* flash the debug LED to tell the world we're in the triggers
	   state.
	*/
	for (count = 0; count < 10; count++)
	{
		DEBUG_LED_ON;
		delay_ms(10);
		DEBUG_LED_OFF;
		delay_ms(90);
	}
	REMOTE_OFF;

	

} /* end triggered state () */


/*
  spook_state
  -----------
  The spook is running.  Do something kind of eerie with the candles.
 */
static void spook_state (void)
{
	uint16_t count;
	
	/* Close the relay to turn on the blacklight */
	RELAY_CLOSED;
	
	/* Stay in this state for about the amt of time it
	   takes the spook to move back and forth on the rope.
	*/
	for (count =0;count<(SPOOK_STATE_SECS);count++)
	{
		int8_t i;
		set_all_leds(1);

		/* 200 ms delay */
		for (i=0;i<10;i++)
		{
			DEBUG_LED_ON;
			delay_ms(5);
			DEBUG_LED_OFF;
			delay_ms(15);
		}

		set_all_leds(0);
		delay_ms(800);				
	}

}  /* end spook_state () */


/*
  recover_state
  -------------
  This state is the same as the flicker state, but
  don't check the proximity sensor.

  The DEBUG LED flashes once per second in recover state.
	
*/
static void recover_state (void)
{
	uint8_t recover_secs_left = RECOVER_STATE_SECS;
/* ------------------------------------------------------------------------ */
	
	RELAY_OPEN;
	REMOTE_OFF;
	
	
	/* Stay in this state  for the number of seconds specified
	   by the macro RECOVER_STATE_SECS
	 */
	while (recover_secs_left-- > 0)
	{
		uint16_t count = 500;
		
		/* flicker the LEDs for one second */
		
		DEBUG_LED_ON;

		while (count-- > 0)
		{
			flicker_leds();
			delay_ms(1);
		}
		
		DEBUG_LED_OFF;
		count = 500;
		while (count-- > 0)
		{
			flicker_leds();
			delay_ms(1);
		}
		
	}
	
} /* end recover_state() */


/*
  main
  ----
  main loop of the program
  
 */
int main(void)
{
	uint16_t count = 0;
/* ----------------------------------------------------------------------- */
	
	/* INITIALIZE */
	init_leds ();
	init_adc ();

	/* enable as output
	   PA7 for DEBUG LED
	   PA6 for relay
	   PA2 for remote control
	*/
	DDRA |= _BV(PA7) | _BV(PA6) | _BV(PA2);

	/* Turn relay off */
	RELAY_OPEN;
	
	/* Blink the DEBUG led to show reset; */

	for (count = 0;count <20;count++)
	{
		DEBUG_LED_ON;
		delay_ms(5);
		DEBUG_LED_OFF;
		delay_ms(20);
	}

	/*  Turn OFF Debug LED*/
	DEBUG_LED_OFF;
	
	/* Simulate candle flicker on LED0
	   LED0 is connected with a 470UF capacitor
	   before the current limiting resistor.
	 */

	while (1)
	{

		flicker_state();

		triggered_state();

		spook_state ();

		recover_state ();

		
	} /* end loop forever */

		
} /* end main () */

