/*********************************************
* Author: Eric Z. Ayers
*
* fauxcandles_flicker_only - Controller for halloween display 2005
* 
* 29 Oct 2005
*
* See http://www.parallexefx.com/  APPNOTE 00001 Faux Candles
*
* This program controls 6 LED candles as described in the ParallexEFX app note
* mentioned above.  There is no trigger, just flickering.  The full version
* isn't working yet, so I did this to make sure I had something
* working for the AHRC meeting Saturday morning.  
*
* PB3 - LED0
* PB4 - LED1
* PB5 - LED2
* PB6 - LED3
* PA4 - LED4
* PA5 - LED5
* 
*
* Debug LED
* PA7 - DEBUG_LED  (with a 220uf cap and a 1K resistor)
*
* Chip type           : AT TINY 26
* Clock frequency     : 8.000000 MHz (internal oscillator)
*
* FUSES:
*   Fuse Low Byte      = 0xe4
*   Fuse High Byte     = 0xf7
*
*********************************************/

/*************************************************************************
 *			Macros
 *************************************************************************/


/* Set CPU frequency for libc routines and macros */
//#define F_CPU 1000000UL  // 1 MHz
//#define F_CPU 14.7456E6
//#define F_CPU 4000000UL  // 4 MHz
//#define F_CPU 8000000UL  // 8 MHz
#define F_CPU 8.0E6  // 8MHz

#define NUM_LEDS	6
#define LED0_PIN PB3
#define LED1_PIN PB4
#define LED2_PIN PB5
#define LED3_PIN PB6
#define LED4_PIN PA4
#define LED5_PIN PA5

/* These parameters control the flickering behavior */
#define DELAY_MS_RANGE 70   // Range of values created
#define MIN_LED_ON       1    // Minimum to leave the LED on
#define MIN_LED_OFF   100  // Minimum time to leave LED off

#define DEBUG_LED_ON	PORTA |= _BV(PA7);
#define DEBUG_LED_OFF	PORTA &= ~(_BV(PA7));


/*************************************************************************
 *			Include Files
 *************************************************************************/

#include <io.h>
#include <stdlib.h>
#include <avr/delay.h>
#include <avr/signal.h>
#include <avr/interrupt.h>

/*************************************************************************
 *			Static Data
 *************************************************************************/

static	int16_t	delay_led_secs[NUM_LEDS];  /* Array containing the # of secs
					      to delay.
					      If # secs > 0, led  is on.  
					      If # secs < 0, led  is off.
					   */

/*************************************************************************
 *			Function Prototypes
 *************************************************************************/

static void delay_ms (uint16_t ms);
static void init_leds (void);
static void set_all_leds (uint8_t value);
static void set_led (int8_t led_number, uint8_t value);

	
/*************************************************************************
 *			Functions
 *************************************************************************/

/*
  delay_ms
  --------
  Delay up to 2^16 milliseconds using the avr libc delay functions
*/
static void delay_ms (uint16_t ms)
{
	for (;ms>0;ms--)
		_delay_ms(1);
}


/*
  Set the LED pins as outputs for their respective ports.w
  Turn off all LEDs.
 */
static void init_leds ( void )
{
	/* Set the pins for LEDs to be outputs */
	DDRB |= _BV(LED0_PIN) | _BV(LED1_PIN) | _BV(LED2_PIN) | _BV(LED3_PIN);
	DDRA |= _BV(LED4_PIN) | _BV(LED5_PIN);
	
	/* Set the output values for the pins to low */
	set_all_leds (0);
}

/*
  set_all_leds
  ------------
  Set all LEDS to the same value
  
  value:    > 0  set all LEDS on,  0 set all LEDS off
  
*/
static void set_all_leds (uint8_t value)
{
	if (value > 0)
	{
		PORTB |= _BV(LED0_PIN) | _BV(LED1_PIN) | _BV(LED2_PIN) |
			_BV(LED3_PIN);
		PORTA |= _BV(LED4_PIN) | _BV(LED5_PIN);
	}
	else
	{
		PORTB &= ~(_BV(LED0_PIN) | _BV(LED1_PIN) | _BV(LED2_PIN)
			   | _BV(LED3_PIN));
		PORTA &= ~(_BV(LED4_PIN) | _BV(LED5_PIN));
	}
}

/*
  set_led
  -------
  Turn a single LED on or off
  
  led_number:  pass a number 0 - 6 to set the specified LED
  value:       > 0 Turn LED on, 0 turn LED off
*/
static void set_led
(
	int8_t led_number,
	uint8_t value
)
{
#define SET_LED(portnum, pin) 		   \
	if (value > 0) {	           \
		(portnum) |= _BV((pin));   \
	} else { 			   \
		(portnum) &= ~(_BV((pin)));\
	}
	
	switch (led_number)
	{
	case 0:
		SET_LED(PORTB, LED0_PIN);
		break;
	case 1:
		SET_LED(PORTB, LED1_PIN);
		break;		
	case 2:
		SET_LED(PORTB, LED2_PIN);
		break;		
	case 3:
		SET_LED(PORTB, LED3_PIN);
		break;		
	case 4:
		SET_LED(PORTA, LED4_PIN);
		break;		
	case 5:
		SET_LED(PORTA, LED5_PIN);
		break;
	}
	
#undef SET_LED
} /* end set_led () */


/* update_leds
   -----------

   Simulate candle flicker on LED0
   LED0 is connected with a 470UF capacitor
   before the current limiting resistor.
*/
void update_leds (void)
{
	uint16_t count;
/* ------------------------------------------------------------------------ */
	
	for (count=0;count<NUM_LEDS;count++)
	{
		if (delay_led_secs[count] > 0)
		{
			/* The LED is currently on.  */
			set_led(count, 1); 
			delay_led_secs[count]--;
			
			if (delay_led_secs[count]==0)
			{
				/* Set the delay for the led to be
				   off
				*/
				delay_led_secs[count]=
					-(((random() % DELAY_MS_RANGE) * 2) + MIN_LED_OFF);
				/* Turn off led */
				set_led(count, 0);
				
				/* go to the top of the loop */
				continue;
			}
		}
		
		
		else if (delay_led_secs[count] < 0)
		{
			/* the LED is currently off */
			set_led(count, 0); 
			delay_led_secs[count]++;
			
			if (delay_led_secs[count] == 0)
			{
				/* Generate a random value
				   between MIN and (MAX_DELAY +MIN)
				*/
				delay_led_secs[count]=
					(random() % DELAY_MS_RANGE)
					+MIN_LED_ON;
				
				/* Turn on led */
				set_led(count, 1); // turn on led
				
				/* go to the top of the loop
				*/
				continue;
				
			}
		}
		
		
	} /* end for each LED */
	
}
/*
  main
  ----
  main loop of the program
  
 */
int main(void)
{
	uint16_t count = 0;

/* ----------------------------------------------------------------------- */
	
	/* INITIALIZE */
	init_leds ();
	
	/* enable PA7 as output */
	DDRA |= _BV(PA7);
	
	
	/* Blink the DEBUG led to show reset; */

	for (count = 0;count <20;count++)
	{
		DEBUG_LED_ON;
		delay_ms(5);
		DEBUG_LED_OFF;
		delay_ms(20);
	}

	/*  Turn OFF Debug LED*/
	DEBUG_LED_OFF;

	

	for (count=0;count<NUM_LEDS;count++)
	{
		delay_led_secs[count]=-1;
	}
	
	while (1)
	{
		count++;

		/* Blink the debug LED 2x/second */
		if(count <1000)
		{
			DEBUG_LED_OFF;
		}
		else if (count > 2000)
		{
			count =0;
		}
		else
		{
			DEBUG_LED_ON;
		}
			
		update_leds();

		/* delay before invoking the flicker again */
		delay_ms(1);

		
	} /* end loop forever */
		
} /* end main () */

