/*********************************************
* Author: Eric Z. Ayers
*
* test_beam_sensor- part of fauxcandles Controller for halloween display 2005
*
* See http://www.parallexefx.com/  APPNOTE 00001 Faux Candles
*
*   An IR photodetector on one pin to detect the presence of an
*     unsuspecting trick-or-treater breaking a flashlight beam.
*
* Flashlight beam to IR detector diode
* PA0 - Read with ADC
*
* Debug LED
* PA7 - DEBUG_LED  (with a 220uf cap and a 1K resistor)
*
* Chip type           : AT TINY 26
* Clock frequency     : 8.000000 MHz
*
*********************************************/

/*************************************************************************
 *			Macros
 *************************************************************************/


/* Set CPU frequency for libc routines and macros */
//#define F_CPU 1000000UL  // 1 MHz
//#define F_CPU 14.7456E6
//#define F_CPU 4000000UL  // 4 MHz
//#define F_CPU 8000000UL  // 8 MHz
#define F_CPU 8.0E6  // 8MHz


/* Macros to control the output devices */
#define DEBUG_LED_ON	PORTA |= _BV(PA7);
#define DEBUG_LED_OFF	PORTA &= ~(_BV(PA7));


// Any reading below this from the ADC,  interpret as someone's nearby
#define BEAM_TRIP_POINT  160 /* .8 volts */
#define MIN_BEAM_TRIPS   50  /* you must trip the beam for 50 ms */

/*************************************************************************
 *			Include Files
 *************************************************************************/

#include <io.h>
#include <stdlib.h>
#include <avr/delay.h>
#include <avr/signal.h>
#include <avr/interrupt.h>

/*************************************************************************
 *			Static Data
 *************************************************************************/
static int num_consecutive_trips = 0;



/*************************************************************************
 *			Function Prototypes
 *************************************************************************/

static void delay_ms (uint16_t ms);
static void init_adc (void);
static uint16_t do_adc (int8_t pin_number);
static int8_t check_beam_sensor (void);


	
/*************************************************************************
 *			Functions
 *************************************************************************/

/*
  delay_ms
  --------
  Delay up to 2^16 milliseconds using the avr libc delay functions
*/
static void delay_ms (uint16_t ms)
{
	for (;ms>0;ms--)
		_delay_ms(1);
}


/* init_adc
   --------
 */
static void init_adc (void )
{
	/* Turn on the ADC */
	ADCSR |= _BV(ADEN);

	/* Set the ADC prescaler /32 */
	ADCSR = 0x85;
}

/*
  do_adc
  ------
  Peform a single adc conversion, flip the adc off before exiting.
  The ADC is 10 bit, so we will get a value between 0 and 1023

  pin_number: ADC pin to read.  Pass a value between 0 & 7
 */
static uint16_t do_adc (int8_t pin_number )
{
	
	/* Configure to A/D Converter for ADC0 input, AVCC 
	   right adjust.
	    SEL MUX
	   0b00 00 <xxx> 
	   <xxx> valid pin numbers are 0-7 
	*/
	DDRA &=~(_BV(pin_number));
	ADMUX = (pin_number);

	/* Start the conversion */
	ADCSR |= _BV(ADSC);
	
	/* Wait for the conversion to finish */
	while (!(ADCSR & _BV(ADSC)))
		; /* do nothing */

	/* Our result is waiting for us in ADCL and ADCH */
	return (ADCL + (((uint16_t)ADCH) <<8));

}


/*
   check_beam_sensor
   -----------------

   The sensor samples the output of the sensor with the ADC which
   returns a value between 0 an 1024.  0 == 0V, 1024 == 5V.
   

   returns: 1 if the proximity sensor detects something 
            0 if not.
   
 */
static int8_t check_beam_sensor ( void )
{
	static int16_t
		num_beam_trips = 0;
/* ------------------------------------------------------------------------- */
	
	int16_t adc_result = do_adc(PA0);

	/* Debounce the input - make sure we don't trigger on some
	   kind of noise.
	*/
	if (adc_result < BEAM_TRIP_POINT)
	{
		num_beam_trips++;
		DEBUG_LED_ON;
	}
	else
	{
		num_beam_trips = 0;
	}

	if (num_beam_trips >= MIN_BEAM_TRIPS)
	{
		/* reset for next time */
		num_beam_trips = 0;
		return 1;
	}
	
	return 0;
	
} /* end check_beam_sensor */




/*
  main
  ----
  main loop of the program
  
 */
int main(void)
{
	uint16_t count = 0;
/* ----------------------------------------------------------------------- */
	
	/* INITIALIZE */
	init_adc ();

	/* enable PA7 as output for DEBUG LED */
	DDRA |= _BV(PA7);
	
	
	/* Blink the DEBUG led to show reset; */

	while (1)
	{
		if (check_beam_sensor())
		{
			DEBUG_LED_ON;
		}
		else
		{
			DEBUG_LED_OFF;
		}
		delay_ms(1);
	}
		
} /* end main () */

